@@ -9,11 +9,11 @@ import random
 import sys
 from argparse import SUPPRESS, Action, ArgumentDefaultsHelpFormatter, ArgumentError, ArgumentParser, Namespace
 from pathlib import Path
-from typing import TYPE_CHECKING, Any, Callable, Dict, List, Literal, Optional, Sequence, Tuple, Type, TypeVar, cast
+from types import UnionType
+from typing import TYPE_CHECKING, Any, Literal, TypeVar, cast
 
 from colorama import Fore
 
-from tox.config.loader.str_convert import StrConvert
 from tox.plugin import NAME
 from tox.util.ci import is_ci
 
@@ -26,6 +26,8 @@ else:  # pragma: <3.11 cover
     from typing_extensions import Self
 
 if TYPE_CHECKING:
+    from collections.abc import Callable, Sequence
+
     from tox.session.state import State
 
 
@@ -51,7 +53,7 @@ class ArgumentParserWithEnvAndConfig(ArgumentParser):
                 outcome = self.file_config.get(key, of_type=of_type)
             if outcome is not None:
                 action.default, default_value = outcome
-                action.default_source = default_value  # type: ignore[attr-defined]
+                action.default_source = default_value  # ty: ignore[unresolved-attribute] # dynamic attr for HelpFormatter
         if isinstance(action, argparse._SubParsersAction):  # noqa: SLF001
             for values in action.choices.values():
                 if not isinstance(values, ToxParser):  # pragma: no cover
@@ -64,7 +66,7 @@ class ArgumentParserWithEnvAndConfig(ArgumentParser):
         of_type: type[Any] | None = getattr(action, "of_type", None)
         if of_type is None:
             if isinstance(action, argparse._AppendAction):  # noqa: SLF001
-                of_type = List[action.type]  # type: ignore[name-defined]
+                of_type = list[action.type]  # ty: ignore[invalid-type-form] # runtime generic from argparse action type
             elif isinstance(action, argparse._StoreAction) and action.choices:  # noqa: SLF001
                 loc = locals()
                 loc["Literal"] = Literal
@@ -78,7 +80,7 @@ class ArgumentParserWithEnvAndConfig(ArgumentParser):
                 raise TypeError(action)
         return of_type
 
-    def parse_args(  # type: ignore[override] # avoid defining all overloads
+    def parse_args(  # avoid defining all overloads
         self,
         args: Sequence[str] | None = None,
         namespace: Namespace | None = None,
@@ -89,7 +91,7 @@ class ArgumentParserWithEnvAndConfig(ArgumentParser):
                 f"unrecognized arguments: {' '.join(argv)}\n"
                 "hint: if you tried to pass arguments to a command use -- to separate them from tox ones",
             )
-        return res
+        return cast("Namespace", res)
 
 
 class HelpFormatter(ArgumentDefaultsHelpFormatter):
@@ -123,19 +125,19 @@ class Parsed(Namespace):
 
     @property
     def verbosity(self) -> int:
-        """:return: reporting verbosity"""
+        """:returns: reporting verbosity"""
         result: int = max(self.verbose - self.quiet, 0)
         return result
 
     @property
     def is_colored(self) -> bool:
-        """:return: flag indicating if the output is colored or not"""
+        """:returns: flag indicating if the output is colored or not"""
         return cast("bool", self.colored == "yes")
 
     exit_and_dump_after: int
 
 
-ArgumentArgs = Tuple[Tuple[str, ...], Optional[Type[Any]], Dict[str, Any]]
+ArgumentArgs = tuple[tuple[str, ...], type[Any] | UnionType | None, dict[str, Any]]
 
 
 class ToxParser(ArgumentParserWithEnvAndConfig):
@@ -230,7 +232,7 @@ class ToxParser(ArgumentParserWithEnvAndConfig):
             help="set PYTHONHASHSEED to SEED before running commands. Defaults to a random integer in the range "
             "[1, 4294967295] ([1, 1024] on Windows). Passing 'notset' suppresses this behavior.",
             action=SeedAction,
-            of_type=Optional[int],  # type: ignore[arg-type]
+            of_type=int | None,
             default=hashseed_default,
             dest="hash_seed",
         )
@@ -239,7 +241,7 @@ class ToxParser(ArgumentParserWithEnvAndConfig):
             dest="discover",
             nargs="+",
             metavar="path",
-            of_type=List[str],
+            of_type=list[str],
             help="for Python discovery first try these Python executables",
             default=[],
         )
@@ -271,16 +273,16 @@ class ToxParser(ArgumentParserWithEnvAndConfig):
                 excl.append((e_kwargs, arguments))
                 res_excl = prev_excl(**kwargs)
                 prev_add_arg = res_excl.add_argument
-                res_excl.add_argument = add_argument  # type: ignore[method-assign]
+                res_excl.add_argument = add_argument  # ty: ignore[invalid-assignment] # wrapping to record args
                 return res_excl
 
             prev_excl = result.add_mutually_exclusive_group
-            result.add_mutually_exclusive_group = add_mutually_exclusive_group  # type: ignore[method-assign]
+            result.add_mutually_exclusive_group = add_mutually_exclusive_group  # ty: ignore[invalid-assignment] # wrapping to record exclusions
             excl: list[tuple[dict[str, Any], list[ArgumentArgs]]] = []
             self._groups.append((args, kwargs, excl))
         return result
 
-    def add_argument(self, *args: str, of_type: type[Any] | None = None, **kwargs: Any) -> Action:
+    def add_argument(self, *args: str, of_type: type[Any] | UnionType | None = None, **kwargs: Any) -> Action:
         result = super().add_argument(*args, **kwargs)
         if self.of_cmd is None and result.dest != "help":
             self._arguments.append((args, of_type, kwargs))
@@ -288,7 +290,7 @@ class ToxParser(ArgumentParserWithEnvAndConfig):
                 for parser in {id(v): v for k, v in self._cmd.choices.items()}.values():
                     parser.add_argument(*args, of_type=of_type, **kwargs)
         if of_type is not None:
-            result.of_type = of_type  # type: ignore[attr-defined]
+            result.of_type = of_type  # ty: ignore[unresolved-attribute] # dynamic attr read by get_type
         return result
 
     @classmethod
@@ -310,7 +312,7 @@ class ToxParser(ArgumentParserWithEnvAndConfig):
         add_core_arguments(self)
         self.fix_defaults()
 
-    def parse_known_args(  # type: ignore[override]
+    def parse_known_args(
         self,
         args: Sequence[str] | None = None,
         namespace: Parsed | None = None,
@@ -357,11 +359,12 @@ def add_verbosity_flags(parser: ArgumentParser) -> None:
 
 
 def add_color_flags(parser: ArgumentParser) -> None:
-    converter = StrConvert()
     if os.environ.get("NO_COLOR", ""):
         color = "no"
-    elif converter.to_bool(os.environ.get("FORCE_COLOR", "")):
+    elif os.environ.get("FORCE_COLOR", ""):
         color = "yes"
+    elif (tty_compat := os.environ.get("TTY_COMPATIBLE", "")) in {"0", "1"}:
+        color = "yes" if tty_compat == "1" else "no"
     elif os.environ.get("TERM", "") == "dumb":
         color = "no"
     else:
@@ -403,7 +406,7 @@ def add_core_arguments(parser: ArgumentParser) -> None:
         metavar="file",
         default=None,
         type=Path,
-        of_type=Optional[Path],
+        of_type=Path | None,
         help="configuration file/folder for tox (if not specified will discover one)",
     )
     parser.add_argument(
@@ -412,7 +415,7 @@ def add_core_arguments(parser: ArgumentParser) -> None:
         metavar="dir",
         default=None,
         type=Path,
-        of_type=Optional[Path],
+        of_type=Path | None,
         help="tox working directory (if not specified will be the folder of the config file)",
     )
     parser.add_argument(
@@ -421,7 +424,7 @@ def add_core_arguments(parser: ArgumentParser) -> None:
         metavar="dir",
         default=None,
         type=Path,
-        of_type=Optional[Path],
+        of_type=Path | None,
         help="project root directory (if not specified will be the folder of the config file)",
     )
 
