@@ -14,6 +14,7 @@ from tox.report import HandledError
 
 if TYPE_CHECKING:
     from configparser import ConfigParser, SectionProxy
+    from types import UnionType
 
     from tox.config.loader.convert import Factory
     from tox.config.loader.section import Section
@@ -60,7 +61,7 @@ class IniLoader(StrConvert, Loader[str]):
     def build(  # noqa: PLR0913
         self,
         key: str,
-        of_type: type[V],
+        of_type: type[V] | UnionType,
         factory: Factory[V],
         conf: Config | None,
         raw: str,
@@ -86,7 +87,7 @@ class IniLoader(StrConvert, Loader[str]):
         prepared = replacer(raw, args) if not delay_replace else raw
         converted = self.to(prepared, of_type, factory)
         if delay_replace:
-            converted.use_replacer(replacer, args)  # type: ignore[attr-defined] # this can be only set_env that has it
+            converted.use_replacer(replacer, args)  # this can be only set_env that has it
         return converted
 
     def found_keys(self) -> set[str]:
