@@ -4,30 +4,32 @@ from __future__ import annotations
 
 import logging
 import os
-import random
-import sys
 import time
 from argparse import Action, ArgumentError, ArgumentParser, Namespace
 from concurrent.futures import CancelledError, Future, ThreadPoolExecutor, as_completed
+from fnmatch import fnmatchcase
 from pathlib import Path
 from signal import SIGINT, Handlers, signal
 from threading import Event, Thread
-from typing import TYPE_CHECKING, Any, Iterator, Optional, Sequence, cast
+from typing import TYPE_CHECKING, Any, cast
+from urllib.parse import unquote, urlparse
+from urllib.request import url2pathname
 
 from colorama import Fore
 
-from tox.config.types import EnvList
 from tox.execute import Outcome
 from tox.journal import write_journal
 from tox.session.cmd.run.single import ToxEnvRunResult, run_one
-from tox.tox_env.runner import RunToxEnv
-from tox.util.ci import is_ci
 from tox.util.graph import stable_topological_sort
 from tox.util.spinner import MISS_DURATION, Spinner
 
 if TYPE_CHECKING:
+    from collections.abc import Iterator, Sequence
+
+    from tox.config.types import EnvList
     from tox.session.state import State
     from tox.tox_env.api import ToxEnv
+    from tox.tox_env.runner import RunToxEnv
 
 
 class SkipMissingInterpreterAction(Action):
@@ -54,25 +56,27 @@ class InstallPackageAction(Action):
     ) -> None:
         if not values:
             raise ArgumentError(self, "cannot be empty")
-        path = Path(cast(str, values)).absolute()
+        raw = cast("str", values)
+        path = self._resolve_path(raw)
         if not path.exists():
             raise ArgumentError(self, f"{path} does not exist")
         if not path.is_file():
             raise ArgumentError(self, f"{path} is not a file")
         setattr(namespace, self.dest, path)
 
+    @staticmethod
+    def _resolve_path(raw: str) -> Path:
+        """Convert a raw string (possibly a ``file:`` URI) to an absolute :class:`~pathlib.Path`."""
+        if raw.startswith("file:"):
+            parsed = urlparse(raw)
+            path = Path(url2pathname(unquote(parsed.path)))
+        else:
+            path = Path(raw)
+        return path.absolute()
 
-def env_run_create_flags(parser: ArgumentParser, mode: str) -> None:  # noqa: C901
+
+def env_run_create_flags(parser: ArgumentParser, mode: str) -> None:
     # mode can be one of: run, run-parallel, legacy, devenv, config
-    if mode not in {"config", "depends"}:
-        parser.add_argument(
-            "--result-json",
-            dest="result_json",
-            metavar="path",
-            of_type=Path,
-            default=None,
-            help="write a JSON file with detailed information about all commands and results involved",
-        )
     if mode not in {"devenv", "depends"}:
         parser.add_argument(
             "-s",
@@ -103,10 +107,17 @@ def env_run_create_flags(parser: ArgumentParser, mode: str) -> None:  # noqa: C9
             "--installpkg",
             help="use specified package for installation into venv, instead of packaging the project",
             default=None,
-            of_type=Optional[Path],
+            of_type=Path | None,
             action=InstallPackageAction,
             dest="install_pkg",
         )
+        parser.add_argument(
+            "--fail-fast",
+            action="store_true",
+            default=False,
+            dest="fail_fast",
+            help="stop execution after the first environment failure",
+        )
     if mode not in {"devenv", "depends"}:
         parser.add_argument(
             "--develop",
@@ -114,51 +125,6 @@ def env_run_create_flags(parser: ArgumentParser, mode: str) -> None:  # noqa: C9
             help="install package in development mode",
             dest="develop",
         )
-    if mode != "depends":
-
-        class SeedAction(Action):
-            def __call__(
-                self,
-                parser: ArgumentParser,  # noqa: ARG002
-                namespace: Namespace,
-                values: str | Sequence[Any] | None,
-                option_string: str | None = None,  # noqa: ARG002
-            ) -> None:
-                if values == "notset":
-                    result = None
-                else:
-                    try:
-                        result = int(cast(str, values))
-                        if result <= 0:
-                            msg = "must be greater than zero"
-                            raise ValueError(msg)  # noqa: TRY301
-                    except ValueError as exc:
-                        raise ArgumentError(self, str(exc)) from exc
-                setattr(namespace, self.dest, result)
-
-        if os.environ.get("PYTHONHASHSEED", "random") != "random":
-            hashseed_default = int(os.environ["PYTHONHASHSEED"])
-        else:
-            hashseed_default = random.randint(1, 1024 if sys.platform == "win32" else 4294967295)  # noqa: S311
-
-        parser.add_argument(
-            "--hashseed",
-            metavar="SEED",
-            help="set PYTHONHASHSEED to SEED before running commands. Defaults to a random integer in the range "
-            "[1, 4294967295] ([1, 1024] on Windows). Passing 'notset' suppresses this behavior.",
-            action=SeedAction,
-            of_type=Optional[int],
-            default=hashseed_default,
-            dest="hash_seed",
-        )
-    parser.add_argument(
-        "--discover",
-        dest="discover",
-        nargs="+",
-        metavar="path",
-        help="for Python discovery first try the Python executables under these paths",
-        default=[],
-    )
     if mode != "depends":
         parser.add_argument(
             "--no-recreate-pkg",
@@ -166,19 +132,6 @@ def env_run_create_flags(parser: ArgumentParser, mode: str) -> None:  # noqa: C9
             help="if recreate is set do not recreate packaging tox environment(s)",
             action="store_true",
         )
-        list_deps = parser.add_mutually_exclusive_group()
-        list_deps.add_argument(
-            "--list-dependencies",
-            action="store_true",
-            default=is_ci(),
-            help="list the dependencies installed during environment setup",
-        )
-        list_deps.add_argument(
-            "--no-list-dependencies",
-            action="store_false",
-            dest="list_dependencies",
-            help="never list the dependencies installed during environment setup",
-        )
     if mode not in {"devenv", "config", "depends"}:
         parser.add_argument(
             "--skip-pkg-install",
@@ -237,7 +190,7 @@ def execute(state: State, max_workers: int | None, has_spinner: bool, live: bool
     state.envs.ensure_only_run_env_is_active()
     to_run_list: list[str] = list(state.envs.iter())
     for name in to_run_list:
-        cast(RunToxEnv, state.envs[name]).mark_active()
+        cast("RunToxEnv", state.envs[name]).mark_active()
     previous, has_previous = None, False
     try:
         spinner = ToxSpinner(has_spinner, state, len(to_run_list))
@@ -261,16 +214,17 @@ def execute(state: State, max_workers: int | None, has_spinner: bool, live: bool
                 if canceled is False and not future.done():  # pragma: no branch
                     tox_env.interrupt()
             done.wait()
-            # workaround for https://bugs.python.org/issue45274
-            lock = getattr(thread, "_tstate_lock", None)
-            if lock is not None and lock.locked():  # pragma: no branch
-                lock.release()  # pragma: no cover
-                # calling private method to fix thread state
-                thread._stop()  # type: ignore[attr-defined] # pragma: no cover # noqa: SLF001
             thread.join()
     finally:
         name_to_run = {r.name: r for r in results}
-        ordered_results: list[ToxEnvRunResult] = [name_to_run[env] for env in to_run_list]
+        ordered_results: list[ToxEnvRunResult] = []
+        for env in to_run_list:
+            if env in name_to_run:
+                ordered_results.append(name_to_run[env])
+            else:
+                ordered_results.append(
+                    ToxEnvRunResult(name=env, skipped=True, code=-2, outcomes=[], duration=MISS_DURATION)
+                )
         # write the journal
         write_journal(getattr(state.conf.options, "result_json", None), state._journal)  # noqa: SLF001
         # report the outcome
@@ -299,7 +253,7 @@ class ToxSpinner(Spinner):
         done(result.name)
 
 
-def _queue_and_wait(  # noqa: C901, PLR0913, PLR0915
+def _queue_and_wait(  # noqa: C901, PLR0913, PLR0915, PLR0912
     state: State,
     to_run_list: list[str],
     results: list[ToxEnvRunResult],
@@ -328,9 +282,15 @@ def _queue_and_wait(  # noqa: C901, PLR0913, PLR0915
             try:
                 executor = ThreadPoolExecutor(max_workers=max_workers, thread_name_prefix="tox-driver")
                 env_list: list[str] = []
+                fail_fast_enabled = options.parsed.fail_fast or any(
+                    cast("RunToxEnv", state.envs[env]).conf["fail_fast"] for env in to_run_list
+                )
                 while True:
-                    for env in env_list:  # queue all available
-                        tox_env_to_run = cast(RunToxEnv, state.envs[env])
+                    envs_to_queue = (
+                        env_list[:1] if max_workers == 1 and fail_fast_enabled and not interrupt.is_set() else env_list
+                    )
+                    for env in envs_to_queue:  # queue all available (or one at a time if sequential + fail-fast)
+                        tox_env_to_run = cast("RunToxEnv", state.envs[env])
                         if interrupt.is_set():  # queue the rest as failed upfront
                             tox_env_to_run.teardown()
                             future: Future[ToxEnvRunResult] = Future()
@@ -339,6 +299,7 @@ def _queue_and_wait(  # noqa: C901, PLR0913, PLR0915
                         else:
                             future = executor.submit(_run, tox_env_to_run)
                         future_to_env[future] = tox_env_to_run
+                    env_list = env_list[len(envs_to_queue) :]
 
                     if not future_to_env:
                         result: ToxEnvRunResult | None = None
@@ -359,8 +320,18 @@ def _queue_and_wait(  # noqa: C901, PLR0913, PLR0915
                             )
                         results.append(result)
                         completed.add(result.name)
+                        if (
+                            result.code != Outcome.OK
+                            and not result.ignore_outcome
+                            and (options.parsed.fail_fast or result.fail_fast)
+                        ):
+                            interrupt.set()
+                            env_list = []
+                            for pending_future in list(future_to_env.keys()):
+                                pending_future.cancel()
 
-                    env_list = next(envs_to_run_generator, [])
+                    if not interrupt.is_set() and not env_list:
+                        env_list = next(envs_to_run_generator, [])
                     # if nothing running and nothing more to run we're done
                     final_run = not env_list and not future_to_env
                     if final_run:  # disable report on final env
@@ -392,7 +363,7 @@ def _handle_one_run_done(
 ) -> None:
     success = result.code == Outcome.OK
     spinner.update_spinner(result, success)
-    tox_env = cast(RunToxEnv, state.envs[result.name])
+    tox_env = cast("RunToxEnv", state.envs[result.name])
     if tox_env.journal:  # add overall journal entry
         tox_env.journal["result"] = {
             "success": success,
@@ -432,8 +403,8 @@ def run_order(state: State, to_run: list[str]) -> tuple[list[str], dict[str, set
     to_run_set = set(to_run)
     todo: dict[str, set[str]] = {}
     for env in to_run:
-        run_env = cast(RunToxEnv, state.envs[env])
-        depends = set(cast(EnvList, run_env.conf["depends"]).envs)
-        todo[env] = to_run_set & depends
+        run_env = cast("RunToxEnv", state.envs[env])
+        depends = set(cast("EnvList", run_env.conf["depends"]).envs)
+        todo[env] = {name for dep in depends for name in to_run_set if fnmatchcase(name, dep)} - {env}
     order = stable_topological_sort(todo)
     return order, todo
