@@ -20,6 +20,7 @@ from tox.tox_env.python.pip.pip_install import Pip
 
 if TYPE_CHECKING:
     from virtualenv.create.creator import Creator
+    from virtualenv.create.describe import Describe
     from virtualenv.discovery.py_info import PythonInfo as VirtualenvPythonInfo
     from virtualenv.run.session import Session
 
@@ -146,21 +147,25 @@ class VirtualEnv(Python, ABC):
             is_64=(interpreter.architecture == 64),  # noqa: PLR2004
             platform=interpreter.platform,
             extra={"executable": Path(interpreter.system_executable).resolve()},
+            free_threaded=interpreter.free_threaded,
         )
 
     def prepend_env_var_path(self) -> list[Path]:
         """Paths to add to the executable."""
         # we use the original executable as shims may be somewhere else
-        return list(dict.fromkeys((self.creator.bin_dir, self.creator.script_dir)))
+        creator = cast(
+            "Describe", self.creator
+        )  # all concrete Creator subclasses inherit Describe via VirtualenvBuiltin
+        return list(dict.fromkeys((creator.bin_dir, creator.script_dir)))
 
     def env_site_package_dir(self) -> Path:
-        return cast("Path", self.creator.purelib)
+        return cast("Path", cast("Describe", self.creator).purelib)
 
     def env_python(self) -> Path:
-        return cast("Path", self.creator.exe)
+        return cast("Path", cast("Describe", self.creator).exe)
 
     def env_bin_dir(self) -> Path:
-        return cast("Path", self.creator.script_dir)
+        return cast("Path", cast("Describe", self.creator).script_dir)
 
     @property
     def runs_on_platform(self) -> str:
@@ -174,11 +179,12 @@ class VirtualEnv(Python, ABC):
 
     @classmethod
     def python_spec_for_path(cls, path: Path) -> PythonSpec:
-        """
-        Get the spec for an absolute path to a Python executable.
+        """Get the spec for an absolute path to a Python executable.
 
         :param path: the path investigated
-        :return: the found spec
+
+        :returns: the found spec
+
         """
         info = cls.get_virtualenv_py_info(path)
         return PythonSpec.from_string_spec(
@@ -187,11 +193,12 @@ class VirtualEnv(Python, ABC):
 
     @staticmethod
     def get_virtualenv_py_info(path: Path) -> VirtualenvPythonInfo:
-        """
-        Get the version info for an absolute path to a Python executable.
+        """Get the version info for an absolute path to a Python executable.
 
         :param path: the path investigated
-        :return: the found information (cached)
+
+        :returns: the found information (cached)
+
         """
         return cached_py_info.from_exe(
             cached_py_info.PythonInfo,
