@@ -3,16 +3,23 @@ from __future__ import annotations
 import logging
 import os
 import sys
+from abc import ABC
 from collections import defaultdict
 from contextlib import contextmanager
 from itertools import chain
 from pathlib import Path
 from threading import RLock
-from typing import TYPE_CHECKING, Any, Dict, Generator, Iterator, NoReturn, Optional, Sequence, cast
+from typing import TYPE_CHECKING, Any, Literal, NoReturn, cast
 
 from cachetools import cached
 from packaging.requirements import Requirement
-from pyproject_api import BackendFailed, CmdStatus, Frontend
+from pyproject_api import (
+    BackendFailed,
+    CmdStatus,
+    Frontend,
+    MetadataForBuildEditableResult,
+    MetadataForBuildWheelResult,
+)
 
 from tox.execute.pep517_backend import LocalSubProcessPep517Executor
 from tox.execute.request import StdinSource
@@ -31,6 +38,8 @@ from tox.util.file_view import create_session_view
 from .util import dependencies_with_extras, dependencies_with_extras_from_markers
 
 if TYPE_CHECKING:
+    from collections.abc import Generator, Iterator, Sequence
+
     from tox.config.sets import EnvConfigSet
     from tox.execute.api import ExecuteStatus
     from tox.tox_env.api import ToxEnvCreateArgs
@@ -45,7 +54,7 @@ if sys.version_info >= (3, 11):  # pragma: no cover (py311+)
 else:  # pragma: no cover (py311+)
     import tomli as tomllib
 
-ConfigSettings = Optional[Dict[str, Any]]
+ConfigSettings = dict[str, Any] | None
 
 
 class ToxBackendFailed(Fail, BackendFailed):
@@ -88,8 +97,8 @@ class ToxCmdStatus(CmdStatus):
         return status.outcome.out_err()
 
 
-class Pep517VirtualEnvPackager(PythonPackageToxEnv, VirtualEnv):
-    """local file system python virtual environment via the virtualenv package."""
+class Pep517VenvPackager(PythonPackageToxEnv, ABC):
+    """local file system python virtual environment package builder."""
 
     def __init__(self, create_args: ToxEnvCreateArgs) -> None:
         super().__init__(create_args)
@@ -100,11 +109,28 @@ class Pep517VirtualEnvPackager(PythonPackageToxEnv, VirtualEnv):
         self._package_dependencies: list[Requirement] | None = None
         self._package_name: str | None = None
         self._pkg_lock = RLock()  # can build only one package at a time
-        self.root = self.conf["package_root"]
         self._package_paths: set[Path] = set()
+        self._root: Path | None = None
+
+    @property
+    def root(self) -> Path:
+        if self._root is None:
+            self._root = self.conf["package_root"]
+        return self._root
+
+    @root.setter
+    def root(self, value: Path) -> None:
+        # Recreating the frontend with a new root would orphan the current frontend.backend_executor, if any, making tox
+        # hang upon exit waiting for its threads and subprocesses (#3512).
+        # Therefore, we make sure to close the existing back-end executor in the case of an existing PEP 517 frontend.
+        if self._frontend is not None:
+            self._frontend.backend_executor.close()
+
+        self._root = value
+        self._frontend_ = None  # force recreating the frontend with new root
 
     @staticmethod
-    def id() -> str:  # noqa: A003
+    def id() -> str:
         return "virtualenv-pep-517"
 
     @property
@@ -127,10 +153,33 @@ class Pep517VirtualEnvPackager(PythonPackageToxEnv, VirtualEnv):
             default=lambda conf, name: self.env_dir / "dist",  # noqa: ARG005
             desc="directory where to put project packages",
         )
+        for key in ("sdist", "wheel", "editable"):
+            self._add_config_settings(key)
+        self.conf.add_config(
+            keys=["fresh_subprocess"],
+            of_type=bool,
+            default=self._frontend.backend.split(".")[0] == "setuptools",
+            desc="create a fresh subprocess for every backend request",
+        )
+
+    def _add_config_settings(self, build_type: str) -> None:
+        # config settings passed to PEP-517-compliant build backend https://peps.python.org/pep-0517/#config-settings
+        keys = {
+            "sdist": ["get_requires_for_build_sdist", "build_sdist"],
+            "wheel": ["get_requires_for_build_wheel", "prepare_metadata_for_build_wheel", "build_wheel"],
+            "editable": ["get_requires_for_build_editable", "prepare_metadata_for_build_editable", "build_editable"],
+        }
+        for key in keys.get(build_type, []):
+            self.conf.add_config(
+                keys=[f"config_settings_{key}"],
+                of_type=dict[str, str],
+                default=None,
+                desc=f"config settings passed to the {key} backend API endpoint",
+            )
 
     @property
     def pkg_dir(self) -> Path:
-        return cast(Path, self.conf["pkg_dir"])
+        return cast("Path", self.conf["pkg_dir"])
 
     @property
     def meta_folder(self) -> Path:
@@ -153,6 +202,13 @@ class Pep517VirtualEnvPackager(PythonPackageToxEnv, VirtualEnv):
         self.builds[build_type].append(run_env.conf)
 
     def _setup_env(self) -> None:
+        if self.conf["deps"]:
+            msg = (
+                f"PEP-517 packaging environment {self.conf.name!r} does not support the deps configuration. "
+                f"Build dependencies should be specified in the [build-system] table of pyproject.toml "
+                f"or by the build backend via get_requires_for_build hooks"
+            )
+            raise Fail(msg)
         super()._setup_env()
         if "sdist" in self.call_require_hooks or "external" in self.call_require_hooks:
             self._setup_build_requires("sdist")
@@ -164,7 +220,8 @@ class Pep517VirtualEnvPackager(PythonPackageToxEnv, VirtualEnv):
             self._setup_build_requires("editable")
 
     def _setup_build_requires(self, of_type: str) -> None:
-        requires = getattr(self._frontend, f"get_requires_for_build_{of_type}")().requires
+        settings: ConfigSettings = self.conf[f"config_settings_get_requires_for_build_{of_type}"]
+        requires = getattr(self._frontend, f"get_requires_for_build_{of_type}")(config_settings=settings).requires
         self._install(requires, PythonPackageToxEnv.__name__, f"requires_for_build_{of_type}")
 
     def _teardown(self) -> None:
@@ -196,22 +253,25 @@ class Pep517VirtualEnvPackager(PythonPackageToxEnv, VirtualEnv):
                 "package config for %s is editable, however the build backend %s does not support PEP-660, falling "
                 "back to editable-legacy - change your configuration to it",
                 names,
-                cast(Pep517VirtualEnvFrontend, self._frontend_).backend,
+                cast("Pep517VirtualEnvFrontend", self._frontend_).backend,
             )
             for env in targets:
-                env._defined["package"].value = "editable-legacy"  # type: ignore[attr-defined]  # noqa: SLF001
+                env._defined["package"].value = "editable-legacy"  # noqa: SLF001  # ty: ignore[invalid-assignment] # ConfigDefinition.value is dynamically typed
                 self.builds["editable-legacy"].append(env)
             self._run_state["setup"] = False  # force setup again as we need to provision wheel to get dependencies
             deps = self._load_deps(for_env)
         of_type: str = for_env["package"]
         if of_type == "editable-legacy":
             self.setup()
-            deps = [*self.requires(), *self._frontend.get_requires_for_build_sdist().requires, *deps]
+            config_settings: ConfigSettings = self.conf["config_settings_get_requires_for_build_sdist"]
+            sdist_requires = self._frontend.get_requires_for_build_sdist(config_settings=config_settings).requires
+            deps = [*self.requires(), *sdist_requires, *deps]
             package: Package = EditableLegacyPackage(self.core["tox_root"], deps)  # the folder itself is the package
         elif of_type == "sdist":
             self.setup()
             with self._pkg_lock:
-                sdist = self._frontend.build_sdist(sdist_directory=self.pkg_dir).sdist
+                config_settings = self.conf["config_settings_build_sdist"]
+                sdist = self._frontend.build_sdist(sdist_directory=self.pkg_dir, config_settings=config_settings).sdist
                 sdist = create_session_view(sdist, self._package_temp_path)
                 self._package_paths.add(sdist)
                 package = SdistPackage(sdist, deps)
@@ -223,11 +283,12 @@ class Pep517VirtualEnvPackager(PythonPackageToxEnv, VirtualEnv):
             else:
                 self.setup()
                 method = "build_editable" if of_type == "editable" else "build_wheel"
+                config_settings = self.conf[f"config_settings_{method}"]
                 with self._pkg_lock:
                     wheel = getattr(self._frontend, method)(
                         wheel_directory=self.pkg_dir,
                         metadata_directory=self.meta_folder_if_populated,
-                        config_settings=self._wheel_config_settings,
+                        config_settings=config_settings,
                     ).wheel
                     wheel = create_session_view(wheel, self._package_temp_path)
                     self._package_paths.add(wheel)
@@ -239,7 +300,7 @@ class Pep517VirtualEnvPackager(PythonPackageToxEnv, VirtualEnv):
 
     @property
     def _package_temp_path(self) -> Path:
-        return cast(Path, self.core["temp_dir"]) / "package"
+        return cast("Path", self.core["temp_dir"]) / "package"
 
     def _load_deps(self, for_env: EnvConfigSet) -> list[Requirement]:
         # first check if this is statically available via PEP-621
@@ -272,6 +333,7 @@ class Pep517VirtualEnvPackager(PythonPackageToxEnv, VirtualEnv):
             deps_with_markers=deps_with_markers,
             extras=extras,
             package_name=project.get("name", "."),
+            available_extras=set(optional_deps.keys()),
         )
 
     def _load_deps_from_built_metadata(self, for_env: EnvConfigSet) -> list[Requirement]:
@@ -280,25 +342,29 @@ class Pep517VirtualEnvPackager(PythonPackageToxEnv, VirtualEnv):
         of_type: str = for_env["package"]
         reqs: list[Requirement] | None = None
         name = ""
-        if of_type in ("wheel", "editable"):  # wheel packages
+        available: set[str] | None = None
+        if of_type in {"wheel", "editable"}:  # wheel packages
             w_env = self._wheel_build_envs.get(for_env["wheel_build_env"])
             if w_env is not None and w_env is not self:
                 with w_env.display_context(self._has_display_suspended):
                     if isinstance(w_env, Pep517VirtualEnvPackager):
-                        reqs, name = w_env.get_package_dependencies(for_env), w_env.get_package_name(for_env)
+                        reqs = w_env.get_package_dependencies(for_env)
+                        name = w_env.get_package_name(for_env)
+                        available = w_env.get_package_extras(for_env)
                     else:
                         reqs = []
         if reqs is None:
             reqs = self.get_package_dependencies(for_env)
             name = self.get_package_name(for_env)
+            available = self.get_package_extras(for_env)
         extras: set[str] = for_env["extras"]
-        return dependencies_with_extras(reqs, extras, name)
+        return dependencies_with_extras(reqs, extras, name, available_extras=available)
 
     def get_package_dependencies(self, for_env: EnvConfigSet) -> list[Requirement]:
         with self._pkg_lock:
             if self._package_dependencies is None:  # pragma: no branch
                 self._ensure_meta_present(for_env)
-                requires: list[str] = cast(PathDistribution, self._distribution_meta).requires or []
+                requires: list[str] = cast("PathDistribution", self._distribution_meta).requires or []
                 self._package_dependencies = [Requirement(i) for i in requires]  # pragma: no branch
         return self._package_dependencies
 
@@ -306,31 +372,47 @@ class Pep517VirtualEnvPackager(PythonPackageToxEnv, VirtualEnv):
         with self._pkg_lock:
             if self._package_name is None:  # pragma: no branch
                 self._ensure_meta_present(for_env)
-                self._package_name = cast(PathDistribution, self._distribution_meta).metadata["Name"]
+                self._package_name = cast("PathDistribution", self._distribution_meta).metadata["Name"]
         return self._package_name
 
+    def get_package_extras(self, for_env: EnvConfigSet) -> set[str]:
+        with self._pkg_lock:
+            self._ensure_meta_present(for_env)
+            return set(cast("PathDistribution", self._distribution_meta).metadata.get_all("Provides-Extra") or [])
+
     def _ensure_meta_present(self, for_env: EnvConfigSet) -> None:
         if self._distribution_meta is not None:  # pragma: no branch
             return  # pragma: no cover
         # even if we don't build a wheel we need the requirements for it should we want to build its metadata
-        target = "editable" if for_env["package"] == "editable" else "wheel"
+        target: Literal["editable", "wheel"] = "editable" if for_env["package"] == "editable" else "wheel"
         self.call_require_hooks.add(target)
 
         self.setup()
         hook = getattr(self._frontend, f"prepare_metadata_for_build_{target}")
-        dist_info = hook(self.meta_folder, self._wheel_config_settings).metadata
-        self._distribution_meta = Distribution.at(str(dist_info))
-
-    @property
-    def _wheel_config_settings(self) -> ConfigSettings | None:
-        return {"--build-option": []}
+        config: ConfigSettings = self.conf[f"config_settings_prepare_metadata_for_build_{target}"]
+        result: MetadataForBuildWheelResult | MetadataForBuildEditableResult | None = hook(self.meta_folder, config)
+        if result is None:
+            config = self.conf[f"config_settings_build_{target}"]
+            dist_info_path, _, __ = self._frontend.metadata_from_built(self.meta_folder, target, config)
+            dist_info = str(dist_info_path)
+        else:
+            dist_info = str(result.metadata)
+        self._distribution_meta = Distribution.at(dist_info)
 
     def requires(self) -> tuple[Requirement, ...]:
         return self._frontend.requires
 
 
+class Pep517VirtualEnvPackager(Pep517VenvPackager, VirtualEnv):
+    """local file system python virtual environment via the virtualenv package."""
+
+    @staticmethod
+    def id() -> str:
+        return "virtualenv-pep-517"
+
+
 class Pep517VirtualEnvFrontend(Frontend):
-    def __init__(self, root: Path, env: Pep517VirtualEnvPackager) -> None:
+    def __init__(self, root: Path, env: Pep517VenvPackager) -> None:
         super().__init__(*Frontend.create_args_from_folder(root))
         self._tox_env = env
         self._backend_executor_: LocalSubProcessPep517Executor | None = None
@@ -353,16 +435,18 @@ class Pep517VirtualEnvFrontend(Frontend):
 
     def _send(self, cmd: str, **kwargs: Any) -> tuple[Any, str, str]:
         try:
-            if (
-                cmd in ("prepare_metadata_for_build_wheel", "prepare_metadata_for_build_editable")
-                # given we'll build a wheel we might skip the prepare step
-                and ("wheel" in self._tox_env.builds or "editable" in self._tox_env.builds)
-            ):
+            if self._can_skip_prepare(cmd):
                 return None, "", ""  # will need to build wheel either way, avoid prepare
             return super()._send(cmd, **kwargs)
         except BackendFailed as exception:
             raise exception if isinstance(exception, ToxBackendFailed) else ToxBackendFailed(exception) from exception
 
+    def _can_skip_prepare(self, cmd: str) -> bool:
+        # given we'll build a wheel we might skip the prepare step
+        return cmd in {"prepare_metadata_for_build_wheel", "prepare_metadata_for_build_editable"} and (
+            "wheel" in self._tox_env.builds or "editable" in self._tox_env.builds
+        )
+
     @contextmanager
     def _send_msg(
         self,
@@ -370,21 +454,25 @@ class Pep517VirtualEnvFrontend(Frontend):
         result_file: Path,  # noqa: ARG002
         msg: str,
     ) -> Iterator[ToxCmdStatus]:
-        with self._tox_env.execute_async(
-            cmd=self.backend_cmd,
-            cwd=self._root,
-            stdin=StdinSource.API,
-            show=None,
-            run_id=cmd,
-            executor=self.backend_executor,
-        ) as execute_status:
-            execute_status.write_stdin(f"{msg}{os.linesep}")
-            yield ToxCmdStatus(execute_status)
-        outcome = execute_status.outcome
-        if outcome is not None:  # pragma: no branch
-            outcome.assert_success()
+        try:
+            with self._tox_env.execute_async(
+                cmd=self.backend_cmd,
+                cwd=self._root,
+                stdin=StdinSource.API,
+                show=None,
+                run_id=cmd,
+                executor=self.backend_executor,
+            ) as execute_status:
+                execute_status.write_stdin(f"{msg}{os.linesep}")
+                yield ToxCmdStatus(execute_status)
+            outcome = execute_status.outcome
+            if outcome is not None:  # pragma: no branch
+                outcome.assert_success()
+        finally:
+            if self._tox_env.conf["fresh_subprocess"]:
+                self.backend_executor.close()
 
-    def _unexpected_response(  # noqa: PLR0913
+    def _unexpected_response(
         self,
         cmd: str,
         got: Any,
@@ -421,3 +509,9 @@ class Pep517VirtualEnvFrontend(Frontend):
 @impl
 def tox_register_tox_env(register: ToxEnvRegister) -> None:
     register.add_package_env(Pep517VirtualEnvPackager)
+
+
+__all__ = [
+    "Pep517VenvPackager",
+    "Pep517VirtualEnvPackager",
+]
