@@ -56,6 +56,7 @@ def test_run_sequential_quiet(tox_project: ToxProjectCreator) -> None:
     assert Matches(r"  a: OK \([\d.]+ seconds\)") == reports[-2]
 
 
+@pytest.mark.slow
 @pytest.mark.integration
 def test_result_json_sequential(
     tox_project: ToxProjectCreator,
@@ -160,6 +161,7 @@ def test_rerun_sequential_wheel(tox_project: ToxProjectCreator, demo_pkg_inline:
     result_rerun.assert_success()
 
 
+@pytest.mark.slow
 @pytest.mark.integration
 def test_rerun_sequential_sdist(tox_project: ToxProjectCreator, demo_pkg_inline: Path) -> None:
     proj = tox_project(
@@ -234,6 +236,7 @@ def test_backend_not_found(tox_project: ToxProjectCreator) -> None:
     assert "packaging backend failed (code=-5), with FailedToStart: could not start backend" in result.out, result.out
 
 
+@pytest.mark.slow
 def test_missing_interpreter_skip_on(tox_project: ToxProjectCreator) -> None:
     ini = "[tox]\nskip_missing_interpreters=true\n[testenv]\npackage=skip\nbase_python=missing-interpreter"
     proj = tox_project({"tox.ini": ini})
@@ -243,6 +246,7 @@ def test_missing_interpreter_skip_on(tox_project: ToxProjectCreator) -> None:
     assert "py: SKIP" in result.out
 
 
+@pytest.mark.slow
 def test_missing_interpreter_skip_off(tox_project: ToxProjectCreator) -> None:
     ini = "[tox]\nskip_missing_interpreters=false\n[testenv]\npackage=skip\nbase_python=missing-interpreter"
     proj = tox_project({"tox.ini": ini})
@@ -460,3 +464,89 @@ def test_missing_command_success_if_ignored(tox_project: ToxProjectCreator) -> N
     result = project.run()
     result.assert_success()
     assert "py: command failed but is marked ignore outcome so handling it as success" in result.out
+
+
+def test_fail_fast_cli_flag(tox_project: ToxProjectCreator) -> None:
+    proj = tox_project({
+        "tox.toml": """
+        [env_run_base]
+        package = "skip"
+        [env.env1]
+        commands = [["python", "-c", "print('env1'); exit(0)"]]
+        [env.env2]
+        commands = [["python", "-c", "print('env2'); exit(1)"]]
+        [env.env3]
+        commands = [["python", "-c", "print('env3'); exit(0)"]]
+        """
+    })
+    result = proj.run("r", "-e", "env1,env2,env3", "--fail-fast")
+    assert result.code != 0
+    assert "env1" in result.out
+    assert "env2" in result.out
+    assert "SKIP" in result.out
+
+
+def test_fail_fast_config(tox_project: ToxProjectCreator) -> None:
+    proj = tox_project({
+        "tox.toml": """
+        [env_run_base]
+        package = "skip"
+        [env.env1]
+        commands = [["python", "-c", "exit(0)"]]
+        [env.env2]
+        fail_fast = true
+        commands = [["python", "-c", "exit(1)"]]
+        [env.env3]
+        commands = [["python", "-c", "exit(0)"]]
+        """
+    })
+    result = proj.run("r", "-e", "env1,env2,env3")
+    assert result.code != 0
+    assert "SKIP" in result.out
+
+
+def test_fail_fast_respects_ignore_outcome(tox_project: ToxProjectCreator) -> None:
+    proj = tox_project({
+        "tox.toml": """
+        [env_run_base]
+        package = "skip"
+        [env.env1]
+        commands = [["python", "-c", "exit(0)"]]
+        [env.env2]
+        ignore_outcome = true
+        commands = [["python", "-c", "exit(1)"]]
+        [env.env3]
+        commands = [["python", "-c", "exit(0)"]]
+        """
+    })
+    result = proj.run("r", "-e", "env1,env2,env3", "--fail-fast")
+    assert result.code == 0
+    assert "env1" in result.out
+    assert "env2" in result.out
+    assert "env3" in result.out
+    assert "IGNORED FAIL" in result.out
+
+
+def test_fail_fast_parallel_mode(tox_project: ToxProjectCreator) -> None:
+    proj = tox_project({
+        "tox.toml": """
+        [env_run_base]
+        package = "skip"
+        [env.env1]
+        commands = [["python", "-c", "exit(0)"]]
+        [env.env2]
+        depends = ["env1"]
+        commands = [["python", "-c", "exit(1)"]]
+        [env.env3]
+        depends = ["env2"]
+        commands = [["python", "-c", "exit(0)"]]
+        [env.env4]
+        depends = ["env3"]
+        commands = [["python", "-c", "exit(0)"]]
+        """
+    })
+    result = proj.run("p", "-e", "env1,env2,env3,env4", "--fail-fast")
+    assert result.code != 0
+    assert "env1" in result.out
+    assert "env2" in result.out
+    assert "SKIP" in result.out
