@@ -82,7 +82,6 @@ def test_package_root_via_testenv(tox_project: ToxProjectCreator, demo_pkg_inlin
     ("conf", "extra", "deps"),
     [
         pytest.param("[project]", "", [], id="no_deps"),
-        pytest.param("[project]", "alpha", [], id="no_deps_with_extra"),
         pytest.param("[project]\ndependencies=['B', 'A']", "", ["A", "B"], id="deps"),
         pytest.param(
             "[project]\ndependencies=['A']\noptional-dependencies.alpha=['B']\noptional-dependencies.beta=['C']",
@@ -98,9 +97,9 @@ def test_package_root_via_testenv(tox_project: ToxProjectCreator, demo_pkg_inlin
         ),
         pytest.param(
             "[project]\ndependencies=['A']\noptional-dependencies.alpha=[]",
-            "alpha,beta",
+            "alpha",
             ["A"],
-            id="deps_with_one_empty_extra",
+            id="deps_with_empty_extra",
         ),
         pytest.param(
             "[project]\ndependencies=['A']\ndynamic=['optional-dependencies']",
@@ -187,19 +186,41 @@ def test_pyproject_deps_from_static(
         assert expected_args == args
 
 
+def test_pyproject_invalid_extra_static(tox_project: ToxProjectCreator, demo_pkg_inline: Path) -> None:
+    conf = "[project]\nname='demo'\ndependencies=['A']\noptional-dependencies.alpha=['B']"
+    toml = f"{(demo_pkg_inline / 'pyproject.toml').read_text()}{conf}"
+    proj = tox_project({"tox.ini": "[testenv]\nextras=typo", "pyproject.toml": toml}, base=demo_pkg_inline)
+    proj.patch_execute(lambda r: 0 if "install" in r.run_id else None)
+    result = proj.run("r", "--notest")
+    result.assert_failed()
+    assert "extras not found for package demo: typo (available: alpha)" in result.out
+
+
+def test_pyproject_invalid_extra_no_optional_deps(tox_project: ToxProjectCreator, demo_pkg_inline: Path) -> None:
+    conf = "[project]\nname='demo'"
+    toml = f"{(demo_pkg_inline / 'pyproject.toml').read_text()}{conf}"
+    proj = tox_project({"tox.ini": "[testenv]\nextras=alpha", "pyproject.toml": toml}, base=demo_pkg_inline)
+    proj.patch_execute(lambda r: 0 if "install" in r.run_id else None)
+    result = proj.run("r", "--notest")
+    result.assert_failed()
+    assert "extras not found for package demo: alpha (available: none)" in result.out
+
+
 @pytest.mark.parametrize(
-    ("metadata", "dynamic", "deps"),
+    ("metadata", "dynamic", "extra", "deps"),
     [
-        pytest.param("Requires-Dist: A", "['dependencies']", ["A"], id="deps"),
+        pytest.param("Requires-Dist: A", "['dependencies']", "", ["A"], id="deps"),
         pytest.param(
-            "Requires-Dist: A\nRequires-Dist: B;extra=='alpha'",
+            "Provides-Extra: alpha\nRequires-Dist: A\nRequires-Dist: B;extra=='alpha'",
             "['dependencies']",
+            "alpha",
             ["A", "B"],
             id="deps_extra",
         ),
         pytest.param(
-            "Requires-Dist: A\nRequires-Dist: B;extra=='alpha'",
+            "Provides-Extra: alpha\nRequires-Dist: A\nRequires-Dist: B;extra=='alpha'",
             "['optional-dependencies']",
+            "alpha",
             ["A", "B"],
             id="deps_extra_dynamic_opt",
         ),
@@ -211,11 +232,12 @@ def test_pyproject_deps_static_with_dynamic(  # noqa: PLR0913
     monkeypatch: pytest.MonkeyPatch,
     metadata: str,
     dynamic: str,
+    extra: str,
     deps: list[str],
 ) -> None:
     monkeypatch.setenv("METADATA_EXTRA", metadata)
     toml = f"{(demo_pkg_inline / 'pyproject.toml').read_text()}[project]\ndynamic={dynamic}"
-    ini = "[testenv]\nextras=alpha\n[testenv:.pkg]\npass_env=METADATA_EXTRA"
+    ini = f"[testenv]\nextras={extra}\n[testenv:.pkg]\npass_env=METADATA_EXTRA"
     proj = tox_project({"tox.ini": ini, "pyproject.toml": toml}, base=demo_pkg_inline)
 
     execute_calls = proj.patch_execute(lambda r: 0 if "install" in r.run_id else None)
@@ -263,19 +285,26 @@ def test_pyproject_no_build_editable_fallback(tox_project: ToxProjectCreator, de
     assert found_calls == expected_calls
 
 
-@pytest.mark.parametrize("package", ["sdist", "wheel", "editable", "editable-legacy", "skip"])
-def test_project_package_with_deps(tox_project: ToxProjectCreator, demo_pkg_setuptools: Path, package: str) -> None:
+@pytest.mark.parametrize("package", ["sdist", "wheel", "editable", "editable-legacy"])
+def test_pep517_pkg_env_rejects_deps(tox_project: ToxProjectCreator, demo_pkg_setuptools: Path, package: str) -> None:
     ini = f"[testenv]\npackage={package}\n[pkgenv]\ndeps = A"
     proj = tox_project({"tox.ini": ini}, base=demo_pkg_setuptools)
-    execute_calls = proj.patch_execute(lambda r: 0 if "install" in r.run_id else None)
+    proj.patch_execute(lambda r: 0 if "install" in r.run_id else None)
     result = proj.run("r", "--notest")
-    result.assert_success()
-    found_calls = [(i[0][0].conf.name, i[0][3].run_id) for i in execute_calls.call_args_list]
-    if package == "skip":
-        assert (".pkg", "install_deps") not in found_calls
-    else:
-        assert found_calls[0] == (".pkg", "install_requires")
-        assert found_calls[1] == (".pkg", "install_deps")
+    result.assert_failed()
+    assert "does not support the deps configuration" in result.out
+
+
+def test_pep517_pkg_env_rejects_deps_via_testenv(
+    tox_project: ToxProjectCreator,
+    demo_pkg_setuptools: Path,
+) -> None:
+    ini = "[testenv]\npackage=sdist\n[testenv:.pkg]\ndeps = B"
+    proj = tox_project({"tox.ini": ini}, base=demo_pkg_setuptools)
+    proj.patch_execute(lambda r: 0 if "install" in r.run_id else None)
+    result = proj.run("r", "--notest")
+    result.assert_failed()
+    assert "does not support the deps configuration" in result.out
 
 
 def test_pyproject_build_editable_and_wheel(tox_project: ToxProjectCreator, demo_pkg_inline: Path) -> None:
@@ -464,3 +493,25 @@ def test_pyproject_config_settings_editable_legacy(
         "get_requires_for_build_wheel": {"C": "3"},
         "prepare_metadata_for_build_wheel": {"D": "4"},
     }
+
+
+@pytest.mark.integration
+@pytest.mark.usefixtures("enable_pip_pypi_access")
+def test_pyproject_installpkg_pep517_envs(tox_project: ToxProjectCreator, pkg_with_pdm_backend: Path) -> None:
+    # Regression test for #3512
+    tox_ini = """
+    [tox]
+    envlist = dummy1,dummy2
+
+    [testenv:dummy1]
+    commands =
+        python -c print(1)
+
+    [testenv:dummy2]
+    commands =
+        python -c print(42)
+    """
+    sdist = pkg_with_pdm_backend / "dist" / "skeleton-0.1.1337.tar.gz"
+    proj = tox_project({"tox.ini": tox_ini}, base=pkg_with_pdm_backend)
+    result = proj.run("--installpkg", str(sdist))
+    result.assert_success()
